﻿using System;
using System.Drawing;
using System.Runtime.InteropServices;
using System.Windows.Forms;

namespace TestProject
{
    public static class WIN32Helper
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Import
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region DC 구하기 - GetDC(windowHandle)

        /// <summary>
        /// DC 구하기
        /// </summary>
        /// <param name="windowHandle">윈도우 핸들</param>
        /// <returns>디바이스 컨텍스트 핸들</returns>
        [DllImport("user32", ExactSpelling = true, SetLastError = true)]
        public static extern IntPtr GetDC(IntPtr windowHandle);

        #endregion
        #region DC 해제하기 - ReleaseDC(windowHandle, deviceContextHandle)

        /// <summary>
        /// DC 해제하기
        /// </summary>
        /// <param name="windowHandle">윈도우 핸들</param>
        /// <param name="deviceContextHandle">디바이스 컨텍스트 핸들</param>
        /// <returns>처리 결과</returns>
        [DllImport("user32", ExactSpelling = true)]
        public static extern IntPtr ReleaseDC(IntPtr windowHandle, IntPtr deviceContextHandle);

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Private

        #region 데스크톱 윈도우 구하기 - GetDesktopWindow()

        /// <summary>
        /// 데스크톱 윈도우 구하기
        /// </summary>
        /// <returns>데스크톱 윈도우 핸들</returns>
        [DllImport("user32", EntryPoint = "GetDesktopWindow")]
        private static extern IntPtr GetDesktopWindow();

        #endregion
        #region 비트맵 복사하기 - BitBlt(targetDCHandle, targetX, targetY, width, height, sourceDCHandle, sourceX, sourceY, rasterOperationCode

        /// <summary>
        /// 비트맵 복사하기
        /// </summary>
        /// <param name="targetDeviceContextHandle">타겟 디바이스 컨텍스트 핸들</param>
        /// <param name="targetX">타겟 X</param>
        /// <param name="targetY">타겟 Y</param>
        /// <param name="width">너비</param>
        /// <param name="height">높이</param>
        /// <param name="sourceDeviceContextHandle">소스 디바이스 컨텍스트 핸들</param>
        /// <param name="sourceX">소스 X</param>
        /// <param name="sourceY">소스 Y</param>
        /// <param name="rasterOperationCode">래스터 연산 코드</param>
        /// <returns>처리 결과</returns>
        [DllImport("gdi32", ExactSpelling = true)]
        private static extern IntPtr BitBlt
        (
            IntPtr targetDCHandle,
            int    targetX,
            int    targetY,
            int    width,
            int    height,
            IntPtr sourceDCHandle,
            int    sourceX,
            int    sourceY,
            int    rasterOperationCode
        );

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 화면 캡처하기 - CaptureScreen()

        /// <summary>
        /// 화면 캡처하기
        /// </summary>
        /// <returns>화면 캡처 비트맵</returns>
        public static Bitmap CaptureScreen()
        {
            int screenWidth  = Screen.PrimaryScreen.Bounds.Width;
            int screenHeight = Screen.PrimaryScreen.Bounds.Height;

            Bitmap screenBitmap = new Bitmap(screenWidth, screenHeight);

            Graphics screenGraphics = Graphics.FromImage(screenBitmap);

            IntPtr desktopDeviceContextHandle = GetDC(GetDesktopWindow());
            IntPtr screenDeviceContextHandle  = screenGraphics.GetHdc();

            BitBlt
            (
                screenDeviceContextHandle,
                0,
                0,
                screenWidth,
                screenHeight,
                desktopDeviceContextHandle,
                0,
                0,
                0x00CC0020 // SRCCOPY
            );

            ReleaseDC(GetDesktopWindow(), desktopDeviceContextHandle);

            screenGraphics.ReleaseHdc(screenDeviceContextHandle);
            screenGraphics.Dispose();

            return screenBitmap;
        }

        #endregion
    }
}